import json
import requests
import urllib

from uritemplate import expand


class Request(object):

    def __init__(self, method, href, auth, resource):
        self.method = method
        self.href = href
        self.auth = auth
        self.resource = resource

    def _get(self, **kwargs):
        # TODO: Refactor to resolve circular dependency.
        from pluct import resource

        data = self.resource.data
        data.update(kwargs)
        url = expand(self.href, data)
        for var in kwargs.keys():
            if "{{{}}}".format(var) in self.href:
                kwargs.pop(var)
        querystring = urllib.urlencode(kwargs)
        if querystring:
            url += "?{0}".format(querystring)

        response = requests.get(url=url,
                                headers=self.get_headers(),
                                timeout=self.resource.timeout)
        return resource.from_response(response)

    def _post(self, **kwargs):
        # TODO: Refactor to resolve circular dependency.
        from pluct import resource

        self.href = expand(self.href, self.resource.data)
        data = kwargs.pop('data')
        response = requests.post(url=self.href,
                                 data=json.dumps(data),
                                 headers=self.get_headers(),
                                 timeout=self.resource.timeout)
        return resource.from_response(response)

    def _patch(self, **kwargs):
        # TODO: Refactor to resolve circular dependency.
        from pluct import resource

        self.href = expand(self.href, self.resource.data)
        data = kwargs.pop('data')
        response = requests.patch(url=self.href,
                                  data=json.dumps(data),
                                  headers=self.get_headers(),
                                  timeout=self.resource.timeout)
        return resource.from_response(response)

    def _put(self, **kwargs):
        # TODO: Refactor to resolve circular dependency.
        from pluct import resource

        self.href = expand(self.href, self.resource.data)
        data = kwargs.pop('data')
        response = requests.put(url=self.href,
                                data=json.dumps(data),
                                headers=self.get_headers(),
                                timeout=self.resource.timeout)
        return resource.from_response(response)

    def _delete(self, **kwargs):
        # TODO: Refactor to resolve circular dependency.
        from pluct import resource

        self.href = expand(self.href, self.resource.data)
        response = requests.delete(url=self.href,
                                   headers=self.get_headers(),
                                   timeout=self.resource.timeout)
        return resource.from_response(response)

    def get_headers(self):
        header = {
            'content-type': 'application/json'
        }
        if self.auth:
            header['Authorization'] = '{0} {1}'.format(
                self.auth['type'], self.auth['credentials'])

        return header

    @property
    def process(self):
        request_type_by_method = {
            'GET': self._get,
            'POST': self._post,
            'PATCH': self._patch,
            'PUT': self._put,
            'DELETE': self._delete,
        }
        return request_type_by_method[self.method]



# -*- coding: utf-8 -*-


import requests

from jsonschema import validate, SchemaError, ValidationError

from pluct import schema
from pluct.request import Request
from pluct.schema import Schema


def add_methods(resource, s, auth=None):
    for link in getattr(s, "links", []) or []:
        method = link.get("method", "GET")
        href = link.get("href")
        rel = link.get("rel")
        method_class = Request(method, href, auth, resource)
        setattr(resource, rel, method_class.process)


class Resource(object):

    def __init__(self, url, data=None, schema=None,
                 auth=None, response=None, timeout=30):
        self.auth = auth
        self.url = url
        self.data = data
        self.schema = schema
        self.timeout = timeout
        self.response = response
        if self.schema:
            if self.is_valid():
                add_methods(self, self.schema, self.auth)
                self.parse_data()

    def __repr__(self):
        return str(self.data)

    def __iter__(self):
        return iter(self.data)

    def __getitem__(self, attr):
        if attr in self.data:
            return self.data[attr]
        raise KeyError

    def is_valid(self):
        try:
            validate(self.data, self.schema.__dict__)
        except (SchemaError, ValidationError):
            return False
        return True

    def parse_data(self):
        if isinstance(self.data, dict):
            for key, value in self.data.items():
                if isinstance(value, list):
                    data_items = []
                    for item in value:
                        try:
                            prop_items = self.schema.properties[key]['items']
                        except KeyError:
                            break
                        if "$ref" in prop_items:
                            s = schema.get(prop_items['$ref'], self.auth)
                        else:
                            s = Schema(self.url, **prop_items)
                        data_items.append(
                            Resource(
                                self.url,
                                data=item,
                                schema=s,
                            )
                        )
                    self.data[key] = data_items


def get(url, auth=None, timeout=30):
    headers = {
        'content-type': 'application/json'
    }
    if auth:
        headers['Authorization'] = '{0} {1}'.format(
            auth['type'], auth['credentials']
        )
    response = requests.get(url, headers=headers, timeout=timeout)
    return from_response(response, auth)


def from_response(response, auth=None):
    try:
        data = response.json()
    except ValueError:
        data = {}
    return Resource(
        url=response.url,
        auth=auth,
        data=data,
        schema=schema.from_header(response.headers, auth),
        response=response
    )



# -*- coding: utf-8 -*-
#!/usr/bin/env python
import re
from setuptools import setup, find_packages, findall


def parse_requirements(file_name):
    requirements = []
    for line in open(file_name, 'r').read().split('\n'):
        if re.match(r'(\s*#)|(\s*$)', line):
            continue
        if re.match(r'\s*-e\s+', line):
            requirements.append(re.sub(r'\s*-e\s+.*#egg=(.*)$', r'\1', line))
        elif re.match(r'\s*-f\s+', line):
            pass
        else:
            requirements.append(line)
    return requirements


def not_py(file_path):
    return not(file_path.endswith('.py') or file_path.endswith('.pyc'))

core_packages = find_packages()
core_package_data = {}
for package in core_packages:
    package_path = package.replace('.', '/')
    core_package_data[package] = filter(not_py, findall(package_path))

setup(
    name='pluct',
    version='0.5.0',
    description='python client to Jsonschema APIs',
    author='Marcos Daniel Petry',
    author_email='marcospetry@gmail.com',
    url='git@github.com:globocom/pluct.git',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: No Input/Output (Daemon)',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Unix',
        'Programming Language :: Python :: 2.7',
    ],
    test_suite='pluct.tests',
    packages=core_packages,
    package_data=core_package_data,
    include_package_data=True,
    install_requires=parse_requirements('requirements.txt'),
)



# -*- coding: utf-8 -*-


import re

import requests


class Schema(object):
    def __init__(self, url, type=None, title=None, required=None,
                 properties=None, links=None):
        self.url = url
        self.type = type
        if required:
            self.required = required
        self.title = title
        if properties:
            self.properties = properties
        if links:
            self.links = links


def get(url, auth=None):
    headers = {
        'content-type': 'application/json'
    }
    if auth:
        headers['Authorization'] = '{0} {1}'.format(
            auth['type'], auth['credentials']
        )
    data = requests.get(url, headers=headers).json()
    return Schema(
        url=url,
        type=data.get("type"),
        required=data.get("required"),
        title=data.get("title"),
        properties=data.get("properties"),
        links=data.get("links")
    )


def from_header(headers, auth=None):
    p = re.compile(".*profile=([^;]+);?")
    schema_url = p.findall(headers.get('content-type', ''))
    if schema_url:
        return get(schema_url[0], auth)
    return None
